mod pixelflut;

use std::thread::JoinHandle;

use crossbeam::channel::*;
use pixelflut::*;
use std::fs;
use std::path::PathBuf;

enum Message {
    Pixel(u32, u32, image::Rgba<u8>),
    Line(Vec<(u32, u32, image::Rgba<u8>)>),
    Exit,
}

fn main() -> Result<(), std::io::Error> {
    let entries = fs::read_dir("nyancat/").unwrap();
    let mut entries: Vec<PathBuf> = entries.map(|entry| entry.unwrap().path()).collect();
    let mut images: Vec<image::DynamicImage> = Vec::new();
    for entry in entries {
        images.push(image::open(entry).expect("Unable to read image format"));
    }
    //let image = image::open("image.png").expect("Unable to read image format");

    let manager = PixelflutConnectionManager::new("10.3.32.187:1234")?;
    let pool = r2d2::Pool::builder()
        .max_size(64)
        .build(manager)
        .expect("Could not initialize connection pool");

    //let image = image.as_rgba8().expect("Not a valid Image");

    let (sender, receiver) = unbounded::<Message>();
    let mut handles: Vec<JoinHandle<()>> = Vec::new();
    for _ in 0..64 {
        let r2 = receiver.clone();
        let p2 = pool.clone();
        let handle = std::thread::spawn(move || {
            let mut connection = match p2.get() {
                Ok(c) => c,
                Err(_) => return,
            };
            loop {
                match r2.recv().unwrap() {
                    Message::Pixel(x, y, c) => {
                        let _ = connection.set(x, y, c);
                    },
                    Message::Line(l) => {
                        let _ = connection.set_multiple(l);
                    },
                    Message::Exit => return,
                }
            }
        });
        handles.push(handle);
    }

    loop{
        for image in &images {
        for (y, row) in image.to_rgba8().enumerate_rows() {
            let row: Vec<(u32,u32,image::Rgba<u8>)> = row.into_iter().map(|(x,y,c)| (x,y,c.clone())).collect();
            let _ = sender.send(Message::Line(row));
        }
        }
    }

    (0..16).for_each(|_| {
        let _ = sender.send(Message::Exit);
    });

    for thread in handles {
        let _ = thread.join();
    }

    Ok(())
}
